/**
 * Breakpoints
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.io.IOException;
import java.util.*;
import xelfi.top.TopLevel;
import xelfi.editor.Position;
import xelfi.editor.Source;
import xelfi.xedit.JavaEditorFrame;
import xelfi.util.EnhProperties;
import xelfi.design.components.dialogs.XFMessageBox;

/**
 * This is a document class which stores information about all breakpoints.
 */

public class Breakpoints
{
	private Debugger debugger;

	private Breakpoint data[];
	private BreakpointsView view;

	/**
	 * Breakpoints constructor.
	 */
	Breakpoints(Debugger debugger)
	{
		this.debugger = debugger;

		data = new Breakpoint[0];
		view = new BreakpointsView(this);
	}

	/**
	 * This method is called by the BreakpointsView when a breakpoint is chosen.
	 */
	void breakpointChosen(int index)
	{
		try
		{
			Breakpoint breakpoint = getBreakpoint(index);

			// get a source and a position of the breakpoint with the specified index
    		Source s = new Source(breakpoint.getClassName(), false);
			
			Position p = TopLevel.getCurrentSnapshot().getPosition(s, breakpoint.getLineNumber()-1).beginOfLine();

			if (p != null)
				TopLevel.showEditor(s, p, true);
			else
				TopLevel.showEditor(s, true);
		}
		catch (IOException e)
		{
			TopLevel.getOutput().show();
			TopLevel.getOutput().println("source code for class '"+getBreakpoint(index).getClassName()+"' not found");
		}
	}

	/**
	 * Returns a number of breakpoints.
	 */
	int getNumberOfBreakpoints()
	{
		return data.length;
	}

	/**
	 * Returns the breakpoint with the specified index.
	 */
	Breakpoint getBreakpoint(int index) throws ArrayIndexOutOfBoundsException
	{
		return data[index];
	}

	/**
	 * Return the view belonging to this document.
	 */
	public BreakpointsView getView()
	{
		return view;
	}

	/**
	 * Called by the BreakpointsView class after its instance has been shown.
	 */
	void breakpointsViewShown()
	{
		view.update(BreakpointsView.HINT_COMPLETE_UPDATE, -1);
	}

	/**
	 * Updates the view on the document.
	 */
	void updateContent()
	{
		view.update(BreakpointsView.HINT_COMPLETE_UPDATE, -1);
	}

	/**
	 * Gets the (breakpoints) properties.
	 */
	void getProperties(EnhProperties properties)
	{
		int numberOfBreakpoints = properties.getIntProperty("NumberOfBreakpoints", 0);

		data = new Breakpoint[0];
		for(int i = 0; i < numberOfBreakpoints; i++)
		{
			String buffer = properties.getProperty("Breakpoint"+(i+1), "");
			StringTokenizer st = new StringTokenizer(buffer, ":");
			String className;
			String lineNumberString;
			String stateString;

			try
			{
				int lineNumber;
				boolean enabled;

				className = st.nextToken();
				lineNumberString = st.nextToken();
				stateString = st.nextToken();

				lineNumber = Integer.valueOf(lineNumberString).intValue();

				enabled = stateString.compareTo("enabled") == 0;
				if (!enabled && (stateString.compareTo("disabled") != 0))
					break;

				Breakpoint breakpoint = new Breakpoint(className, lineNumber);

				if (!enabled)
					breakpoint.setDisabled();

				addBreakpoint(breakpoint);

			}
			catch(NoSuchElementException e)
			{
				break;
			}
			catch(NumberFormatException e)
			{
				break;
			}
		}
	}

	/**
	 * Puts the (breakpoints) properties.
	 */
	void putProperties(EnhProperties properties)
	{
		int numberOfBreakpoints = properties.getIntProperty("NumberOfBreakpoints", 0);

		for(int i = 0; i < numberOfBreakpoints; i++)
			properties.remove("Breakpoint"+(i+1));

		numberOfBreakpoints = getNumberOfBreakpoints();
		properties.setIntProperty("NumberOfBreakpoints", numberOfBreakpoints);

		for(int i = 0; i < numberOfBreakpoints; i++)
		{
			Breakpoint breakpoint = getBreakpoint(i);
			String buffer = breakpoint.getClassName()+":"+
				breakpoint.getLineNumber()+":";

			if (breakpoint.isEnabled())
				buffer = buffer+"enabled";
			else
				buffer = buffer+"disabled";

			properties.put("Breakpoint"+(i+1), buffer);
		}
	}

	/**
	 * Inserts a new breakpoint on the position with the specified index.
	 */
	void insertBreakpoint(int index, Breakpoint breakpoint) throws ArrayIndexOutOfBoundsException
	{
		int numberOfBreakpoints = getNumberOfBreakpoints();

		for(int i = 0; i < numberOfBreakpoints; i++)
			if (getBreakpoint(i).getClassName().equals(breakpoint.getClassName())
				&& getBreakpoint(i).getLineNumber() == breakpoint.getLineNumber())
				return;

		int newSize = data.length+1;
		Breakpoint newData[] = new Breakpoint[newSize];

		for(int i = 0; i < index; i++)
			newData[i] = data[i];
		newData[index] = breakpoint;
		for(int i = index+1; i < newSize; i++)
			newData[i] = data[i-1];

		data = newData;

		debugger.addBreakpoint(breakpoint);
		view.update(BreakpointsView.HINT_INSERTED, index);
	}

	/**
	 * Add a new breakpoint to the end.
	 */
	void addBreakpoint(Breakpoint breakpoint)
	{
		int numberOfBreakpoints = getNumberOfBreakpoints();

		for(int i = 0; i < numberOfBreakpoints; i++)
			if (getBreakpoint(i).getClassName().equals(breakpoint.getClassName())
				&& getBreakpoint(i).getLineNumber() == breakpoint.getLineNumber())
				return;

		Breakpoint newData[] = new Breakpoint[numberOfBreakpoints+1];

		for(int i = 0; i < numberOfBreakpoints; i++)
			newData[i] = data[i];
		newData[numberOfBreakpoints] = breakpoint;

		data = newData;

		debugger.addBreakpoint(breakpoint);
		view.update(BreakpointsView.HINT_INSERTED, numberOfBreakpoints);
	}

	/**
	 * Remove the breakpoint from the specified index.
	 */
	void removeBreakpoint(int index) throws ArrayIndexOutOfBoundsException, NegativeArraySizeException
	{
		int newSize = getNumberOfBreakpoints()-1;
		Breakpoint newData[] = new Breakpoint[newSize];
		String className = data[index].getClassName();
		int lineNumber = data[index].getLineNumber();

		for(int i = 0; i < index; i++)
			newData[i] = data[i];
		for(int i = index; i < newSize; i++)
			newData[i] = data[i+1];

		data = newData;

		debugger.removeBreakpoint(className, lineNumber);
		view.update(BreakpointsView.HINT_REMOVED, index);
	}

	/**
	 * Enables a breakpoint with the specified index.
	 */
	void enableBreakpoint(int index)
	{
		Breakpoint breakpoint = getBreakpoint(index);

		breakpoint.setEnabled();
		if (!breakpoint.isValid())
			return;

		debugger.addBreakpoint(breakpoint);
		view.update(BreakpointsView.HINT_CHANGED, index);
	}

	/**
	 * Disables a breakpoint with the specified index.
	 */
	void disableBreakpoint(int index)
	{
		Breakpoint breakpoint = getBreakpoint(index);

		breakpoint.setDisabled();
		if (!breakpoint.isValid())
			return;

		debugger.removeBreakpoint(breakpoint.getClassName(), breakpoint.getLineNumber());
		view.update(BreakpointsView.HINT_CHANGED, index);
	}
}
