/**
* Xelfi
*/

package xelfi.design.components;

import java.awt.*;
import java.io.*;
import xelfi.design.components.property.*;
import xelfi.design.forms.XelfiDesignForm;

/**
* XelfiDesignComponent interface is a common interface for components that can be used 
* during Xelfi design time.<p>
* The user who wants to create new component should write the <b> component class </b>.
* Then its subclass - <b> design class </b>- implementing this
* interface must be written.<p>
* The design class is intended only to implement common access to component functionality.
* The component look&feel is to be implemented in the component class.
* The design class will be used during design time only, so the design
* things will not be used in final code (either source or .class).
* <p>
* Some methods of this interface has common implementation (e.g. setBehaviour(), 
* getBehaviour(), handleEvent(), ... - these should be generated by Xelfi or verbatim-copied
* from existing implementations to ensure proper functionality) and some must be 
* written to implement component properties and behaviour (e.g. getPropertyCount(),
* setProperty(), getProperty(), ...).
*
* @version 	1.60, 20 Feb 1997
*		from 1.30 code generation added
*		from 1.40 event properties added
*		from 1.50 method for generating event condition
*		from 1.51 method neededImports
*		from 1.60 setName return value
* @author 	Ian Formanek
*/
public interface XelfiDesignComponent {
	/**
	* Method that allows component to acquire the form that contains it.
	* @param	form	form that contains this component
	*/
	public void setParentForm(XelfiDesignForm form);

	/**
	* Method that allows to acquire the form that contains this component.
	* @return	the form that contains this component
	*/
	public XelfiDesignForm getParentForm();

	/**
	* Method to get number of properties in component. 
	* @return				number of properties that component has
	* @see #getPropertyNames
	* @see #getProperty
	* @see #setProperty
	*/
	public int getPropertyCount();
	
	/**
	* Method to get names of properties in component. 
	* @return				array of all property names
	* @see #getPropertyCount
	* @see #getProperty
	* @see #setProperty
	*/
	public String[] getPropertyNames();
	
	/**
	* Method to get value of given property. 
	* @param	index		the index of property to set
	* @return				the value of given property
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	* @see #setProperty
	* @see #getPropertyCount
	* @see #getPropertyNames
	*/
	public Object getProperty(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	/**
	* Method to get textual reprezentation of value of given property.
	* @param	index		the index of property to set
	* @return				the String reprezentation of value of given property
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	*/
	public String getPropertyString(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	/**
	* Method to get constant that describes what viewer to use in ComponentInspector for
	*	given property (the constants are defined in XFDesignConstants)
	* @param	index		the index of property
	* @return				the constant specifying what viewer to use
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	* @see #getPropertyValuesArray
	*/
	public int getPropertyViewerType(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	
	
	/**
	* Method to get array of textual representation of all possible values for given property.
	* This is only for properties with viewer type PVT_CHOICE.
	* @param 	index	the index of property
	* @return		the array of string value names
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	*			or when the given property does not have a viewer type PVT_CHOICE
	* @see #getPropertyViewerType
	*/
	public String[] getPropertyValueNames(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	
	/**
	* Method to get array of textual representation of all possible values for given property.
	* This is only for properties with viewer type PVT_CHOICE.
	* @param 	index	the index of property
	* @return		the array of possible valuesvalues 
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	*			or when the given property does not have a viewer type PVT_CHOICE
	* @see #getPropertyViewerType
	*/
	public Object[] getPropertyValueValues(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	/**
	* Method to set given property to given value. It returns one of RSP_??? constants defined
	* in XFDesignConstants.
	* @param	index		the index of property to set
	* @param	value		the value to which to set the property
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	* @exception XFPropertyBadValueTypeException
	* 		Should be thrown when the value type is not the same as tpe of the property
	* @return	one of RSP_??? constants defined in XFDesignConstants.
	* @see #getProperty
	* @see #getPropertyCount
	* @see #getPropertyNames
	*/
	public int setProperty(int index, Object value) throws 
	XFPropertyIndexOutOfBoundsException,
	XFPropertyBadValueTypeException ;
	
	
	/**
	* This method should be invoke by parent of component,
	* when parent changes. For example when parent component
	* changes Font, component should be notified.
	* @return	true, if component was repaint, otherwise false.
	*/
	public boolean parentUpdate();

	/**
	* Method to invoke specialInput Dialog on given property. The parent parameter is
	* a parent of the dialog.
	* @param	index		the index of property to set
	* @param	parent	parent Frame of the dialog
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	* @exception XFPropertyNoSpecialInputException
	* 		Should be thrown when the given property does not have specialInput dialog
	*/
	public void specialPropertyInput(Frame parent, int index) throws 
	XFPropertyIndexOutOfBoundsException,
	XFPropertyNoSpecialInputException;	
	
	/**
	* Method to get component's name
	* @return		Component name
	* @see #setDesignName
	*/
	public String getName();
	
	/**
	* Method to set component's name
	* @param	aName	Component name
	* @return	one of RSP_??? constants defined in XFDesignConstants.
	* @see #getName
	*/
	public int setDesignName(String aName);
	
	/**
	* Method to get number of event properties in component.
	* @return				number of event properties that component has
	* @see #getEventNames
	* @see #getEvent
	* @see #setEvent
	*/
	public int getEventCount();
	
	/**
	* Method to get names of event properties in component.
	* @return				array of all event property names
	* @see #getEventCount
	* @see #getEvent
	* @see #setEvent
	*/
	public String[] getEventNames();
	
	/**
	* Method to get name of event handling method of given event property.
	* @param	index		the index of property to set
	* @return				the name of event handling method or null (if not set)
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	* @see #setEvent
	* @see #getEventCount
	* @see #getEventNames
	*/
	public String getEvent(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	/**
	* Method to set name of event handling method of given event property to given value.
	* @param	index		the index of property to set
	* @param	value		the value to which to set the event property
	* @exception XFPropertyIndexOutOfBoundsException
	* 		should be thrown when Index value is out of range 
	*			<tt>((index < 0) || (index > property count-1))</tt>
	* @see #getEvent
	* @see #getEventCount
	* @see #getEventNames
	*/
	public void setEvent(int index, String value) throws 
	XFPropertyIndexOutOfBoundsException;
	
	/**
	* Method to get comonent's icon
	* @return	icon image
	*/
	public Image getIcon();
	
	/**
	* Method to set name of the COMPONENT class (e.g. for XButton extends Button it will 
	* return "Button")
	* @return	name of the component class
	*/
	public String getClassName();
	
	/////////////////////
	// code generation //
	/////////////////////
	
	/**
	* Method to find out what a component needs to import (or what should be generated into
	* imports section of generated source of form that uses this component).
	* It returns array of strings. Each of these strings corresponds with one import line.
	* The string is only what should be inside import (e.g. only "java.awt.*" for "import java.awt.*;")
	* @return		array of strings that appear in the imports
	*/
	public String[] neededImports();

	/**
	* Method that manages selection state of this component.
	* @param	aSelection 	one of constants from XFDesignConstants: SEL_UNSELECT, SEL_SELECT, SEL_MUL_SELECT
	*/
	public void setSelection(int aSelection);

	/**
	* Method that returns current selection state of this component.
	* @return	 	one of constants from XFDesignConstants: SEL_UNSELECT, SEL_SELECT, SEL_MUL_SELECT
	*/
	public int getSelection();

	/**
	* This method of component is called by a Form that contains it for generating 
	* given section of code (parameter what).
	* @param what	constant (defined in XFDesignConstants) specifying the section to generate
	* @return string with generated code
	*/
	
	public String generateCode(int what);

	/**
	* This method of a component is called by a Form that contains it for generating 
	* given event's condition to handleEvent method.
	* Only the condition that is inside brackets of "if" should be generated
	* (e.g. "evt.target == button1" for if (evt.target == button1) { ... )
	* Please note that the name of event variable ("evt" in example) is set in XFDesignOptions.eventVarName
	* @param index	index of event which condition to generate
	* @return string with generated condition
	*/
	public String generateEventCondition(int index) throws 
	XFPropertyIndexOutOfBoundsException;
	
	/////////////////
	// persistence //
	/////////////////
	
	/**
	* This method loads this component's properties from given stream.
	* For XelfiVisualContainer components it also loads all its children.
	* @param stream	input stream to load component from
	* @see #loadFromStream
	*/
	public void loadFromStream(DataInput stream) throws
	IOException;
	
	/**
	* This method saves this component's properties to given stream .
	* For XelfiVisualContainer components it also saves all its children.
	* @param stream	output stream to save component to
	* @see #saveToStream
	*/
	public void saveToStream(DataOutput stream) throws
	IOException;
}	