/**
* Xelfi
*/

package xelfi.design.components;

import java.awt.*;
import xelfi.design.components.property.*;

/**
 * XelfiVisualComponent Interface is a common interface for visual components that can be used 
 * during Xelfi design time.<p>
 * The user who wants to create new component should write the component as descendant
 * from Canvas class (or any subclass of it) - a <b> component class </b>.
 * Then its subclass - <b> design class </b>- implementing this
 * interface must be written.<p>
 * The design class is intended only to implement common access to component functionality.
 * The component look&feel is to be implemented in the component class.
 * The design class will be used during design time only, so the design
 * things will not be used in final code (either source or .class).
 * <p>
 * Some methods of this interface has common implementation (e.g. setBehaviour(), 
 * getBehaviour(), handleEvent(), ... - these should be generated by Xelfi or verbatim-copied
 * from existing implementations to ensure proper functionality) and some must be 
 * written to implement component properties and behaviour (e.g. getPropertyCount(),
 * setProperty(), getProperty(), ...).
 *
 * @version 	1.30, 27 Feb 1997 
 *		from 1.10 behaviour removed, update added
 *		from 1.20 setPosition, setSize return value
 * @author 	Ian Formanek
 */
public interface XelfiVisualComponent extends XelfiDesignComponent {

	/**
	 * Method to get component's position
	 * @return		Component position
	 * @see #setPosition
	 * @see #getSize
	 * @see #setDesignSize
	 */
	public Point getPosition();

	/**
	 * Method to set component's position
	 * @param	x	Component x position
	 * @param	y	Component y position
	 * @return		one of RSP_??? constants defined in XFDesignConstants.
	 * @see #getPosition
	 * @see #getSize
	 * @see #setDesignSize
	 */
	public int setPosition(int x, int y);

	/**
	 * Method to get component's size. It works aside getPreferredSize() to
	 * support absolute positioning. 
	 * It can also work the way that if another Component's property, say "autoSize", is set,
	 * then getSize() returns preferredSize() from the component class, in the other case 
	 * it returns the size previously set by setSize(). 
	 * This ensures that real component implementation has something to
	 * say to component's size.
	 * @return		Component size
	 * @see #setDesignSize
	 * @see #getPosition
	 * @see #setPosition
	 */
	public Dimension getSize();

	/**
	 * Method to get component's size (from the size property) which includes all the 
	 * design things painted around the real component (selection frame, XAWTPanel's 
	 * catch border). This is used by layouts that allow dragging only.
	 * @return		Component size including selection frame, etc.
	 * @see #setDesignSize
	 */
	public Dimension getDesignSize();

	/**
	 * Method to set Component's size.
	 * @param	w	Component width
	 * @param	h	Component height
	 * @return		one of RSP_??? constants defined in XFDesignConstants.
	 * @see #getSize
	 * @see #getPosition
	 * @see #setPosition
	 */
	public int setDesignSize(int w, int h);

	/**
	 * Method for eventHandling during design time.
	 * Common implementation should look like:
	 * <pre>
	 * public boolean handleEvent(Event evt) {
	 *   if (behaviour == XDDESIGN) return false;
	 *   else return super.handleEvent(evt);
	 *   }
	 * </pre>
	 * where behaviour is a private variable of the design class.
	 * @return			same as normal handleEvent method
	 * @see #setBehaviour
	 * @see #getBehaviour
	 * @see #XDDESIGN
	 * @see #XDREAL
	 */
	public boolean handleEvent(Event evt);
}	