/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
/**
* Object that supports drawing of text on an image.
*
*
* @version 1.0
* @author Jaroslav Tulach
*/

class EditorImage extends Block implements ActionPostListener, RedrawListener {
  /** the dimension of a image */
  Dimension imDimension;
  /** the image itself */
  Image imImage;
  /** context to write to */
  Graphics imGraphics;
  /** context of screen */
  Graphics screenGraphics;

  /** the editor to display image to */
  EditorCanvas editor;

  /** redraw flag */
  private int redrawLock = 0;

  /** redraw delayed. Set if a whole redraw should be done
  */
  private boolean redrawDelayed = false;

  /** the first column to display */
  int firstColumn = 0;

  /** delta when we are scroling to left and right */
  private static final int horzDelta = 8;

  /** has already been registered */
  boolean registered = false;

  /** true if the cursor is visible */
  Point cursor;

  /** creates new object
  * @param e the editor to attach to
  */
  EditorImage (EditorCanvas e) {
    editor = e;
    setBegin (begin = e.getDocument ().getBegin ());
    // begin is set to begining of document that is displayed by editor

    // register itself as a block controler
    editor.getDocument ().prependActionPostListener (this);
    editor.getDocument ().addRedrawListener (this);
  };

  /** Removes itself from controlers. And that is why this object can be
  * cleared. Otherwise remains reference from document to this instance.
  */
  public final void close () {
    editor.getDocument ().removeActionPostListener (this);
    editor.getDocument ().removeRedrawListener (this);
  };

  /** Sets new begining of displayed area. The whole screen is redrawn and
  * new end is also set.
  * @return true, if the begin changed
  * @param p the position that drawing should start at
  */
  synchronized final boolean setBegin (Position p) {
    synch ();
    p = p.beginOfLine ();
    if (!p.equals (getBegin ())) {
      begin = p;
      end = getEnd ();
      initialDraw ();
      return true;
    };
    return false;
  };
  
  /** tests if a editor view size and dim of imImage are equal, if not
  * creates new image and graphics ...
  */
  synchronized final void synch () {
    Dimension d = editor.size ();

    try {
      if (
        imDimension == null ||
        d.width != imDimension.width ||
        d.height != imDimension.height
      ) {
        imDimension = d;
        imImage = editor.createImage (d.width, d.height);
        imGraphics = imImage.getGraphics ();
        screenGraphics = editor.getGraphics ();
        setColor (editor.getBackground ());
        fillRect (0, 0, d.width, d.height);

        if (editor.getCursorPos ().less (getBegin ()) || editor.getCursorPos ().greater (getEnd ())) {
          setBegin (editor.getCursorPos ().safeUp (visibleLinesCount () / 2));
        };

        initialDraw ();
        // starts redraw of image
      };
    } catch (NullPointerException ex) {
      // peer has not been created yet
    };
  };

  /** Sets graphic context to informations in DrawInfo.
  * @param di the draw info
  * @param foreground prepare to draw foreground or background
  */
  private void useTextDrawInfo (TextDrawInfo di, boolean foreground) {
    try {
      if (foreground) {
        imGraphics.setColor (di.text);
        screenGraphics.setColor (di.text);
      } else {
        imGraphics.setColor (di.back);
        screenGraphics.setColor (di.back);
      };

      imGraphics.setFont (di.font);
//      editor.setFont (di.font);
      screenGraphics.setFont (di.font);
    } catch (NullPointerException ex) {
    };
  };

  private void setColor (Color c) {
    try {
      imGraphics.setColor (c);
      screenGraphics.setColor (c);
    } catch (NullPointerException ex) {
    };
  };

  /** Sets or resets XOR mode.
  * @param xor if true the XOR mode is set otherwise the normal mode is set
  */
  private void setMode (boolean xor) {
    try {
      if (xor) {
        imGraphics.setXORMode (Color.lightGray);
        screenGraphics.setXORMode (Color.lightGray);
      } else {
        imGraphics.setPaintMode ();
        screenGraphics.setPaintMode ();
      };
    } catch (NullPointerException ex) {
    };
  };

  private void fillRect (int x, int y, int w, int h) {
    try {
      imGraphics.fillRect (x, y, w, h);
      if (redrawLock == 0) {
        screenGraphics.fillRect (x, y, w, h);
      };
    } catch (NullPointerException ex) {
    };
  };

  private FontMetrics getFontMetrics () {
    return imGraphics.getFontMetrics ();
  };

  private void copyArea (int x, int y, int w, int h, int dx, int dy) {
    try {
      imGraphics.copyArea (x, y, w, h, dx, dy);
      if (redrawLock == 0) {
        screenGraphics.copyArea (x, y, w, h, dx, dy);
      };
    } catch (NullPointerException ex) {
    };
  };

  private void drawString (String s, int x, int y) {
    try {
      imGraphics.drawString (s, x, y);
      if (redrawLock == 0) {
        screenGraphics.drawString (s, x, y);
      };
    } catch (NullPointerException ex) {
    };
  };

  /** Gets the image that should contain displayed text
  * @return the image
  */
  public final Image getImage () {
    synch ();
    return imImage;
  };

  /** New begin. Alway at the begin of line.
  * @return the begin of block
  */
  public Position getBegin () {
    return (begin = begin.beginOfLine ());
  };

  /** Gets end of block.
  * @param the end of block
  */
  public Position getEnd () {
    try {
      return getBegin ().down (visibleLinesCount ());
    } catch (EndOfTextException ex) {
      return getBegin ().getDocument ().getEnd ().endOfLine ().next ();
    }
  }

  /** Converts position to a point. A draw can be started at specified position.
  * @param p the position
  * @return the coordinates of the character
  * @exception InvalidPositionException if position is not visible
  */
  Point positionToPoint (Position p) throws InvalidPositionException {
    Dimension d = editor.info.getCharDimension ();
    return new Point (
      (p.getColumn () - firstColumn) * d.width,
      d.height * lineToInt (p.beginOfLine ()) + editor.info.getMaxAscent ()
    );
  };

  /** Converts point to position.
  * @param x the x point
  * @param y the y point
  * @return the position at this point
  */
  Position pointToPosition (int x, int y) throws EndOfTextException {
    Dimension d = editor.info.getCharDimension ();
    int l = y / d.height;
    int c = x / d.width + firstColumn;
    if (c < 0) {
      c = 0;
    };
    return goToLine (l).column (c);
  };

  /** Views specified position. Changes column info.
  * @param p the position to be visible
  */
  void makeHorizontalVisible (Position p) {
    int pos = p.getColumn ();
    Dimension dim = editor.size ();

    if (pos < firstColumn) {
      int n = pos - horzDelta;
      if (n < 0) n = 0;

      int delta = firstColumn - n;
      int o = firstColumn;
      firstColumn = n;

      copyArea (0, 0, dim.width, dim.height,
        delta * editor.info.getCharDimension ().width, 0
      );
      drawColumns (getBegin (), visibleLinesCount (), firstColumn, o);
//      draw ();
    } else {
      Dimension d = editor.info.getCharDimension ();
      int del = editor.size ().width / d.width - 1;
      if (pos - firstColumn > del) {
        pos += horzDelta;
      };

      int delta = pos - firstColumn - del;
      if (delta > 0) {
        firstColumn = pos - del;
        //        draw ();
        int dx = delta * d.width;
        copyArea (dx, 0, dim.width, dim.height,
          -dx, 0
        );
        drawColumns (getBegin (), visibleLinesCount (), pos - delta, pos + 1);
      };
    };
  };

  /** How much lines can fit to whole window
  * @return the number of lines that can fit to displayed area
  */
  int visibleLinesCount () {
    int h = editor.info.getCharDimension ().height;
    int c = editor.size ().height / h;
    return c;
  };

  /** Draws and sets guarded block. */
  synchronized final void initialDraw () {
    setDefaultBackground ();
    redrawLock = 1;
    redrawDelayed = true;
    redrawUnlock (null);
  };

  /** Gets info about how to draw the rest of text. Asks the info callback.
  * @param old the previous text info
  * @return the info
  * @exception InvalidPositionException if the position is not valid
  */
  final TextDrawInfo getTextDrawInfo (TextDrawInfo old) throws InvalidPositionException {
    Position end = old.end;
    TextDrawInfo di = editor.info.getDrawInfo (old);

    if (di.end.lessOrEqual (end)) {
      throw new InvalidPositionException ("");
    } else {
      return di;
    }
  }

  /** Draws a cursor. The character at the cursor position is changed
  * by XOR operation.
  */
  final void drawCursor () {
    int masc = editor.info.getMaxAscent ();
    Dimension d = editor.info.getCharDimension ();
    try {
      if (cursor != null) {
        setColor (Color.red);
        setMode (true);

        fillRect (cursor.x, cursor.y - masc, d.width, d.height);

        setMode (false);
        setColor (Color.white);

        cursor = null;
      } else {
        cursor = positionToPoint (editor.getCursorPos ());
        setColor (Color.red);
        setMode (true);

        fillRect (cursor.x, cursor.y - masc, d.width, d.height);

        setMode (false);
        setColor (Color.white);
      };
    } catch (InvalidPositionException ex) {
    };
  };

  /** Draws a part of text. On one line.
  * @param p the position to begin display from
  * @param info the info about drawing color, font ...
  * @param x the x position on screen to draw at
  * @param y the y position
  * @return draw info to be used on next line
  */
  final synchronized TextDrawInfo drawLine (
    Position p, TextDrawInfo drawInfo, int x, int y
  ) {
    return drawLine (p, null, drawInfo, x, y);
  };

  /** Draws a part of text. On one line.
  * @param p the position to begin display from
  * @param e the position to display to (on the same line)
  * @param ee the position to end at
  * @param info the info about drawing color, font ...
  * @param x the x position on screen to draw at
  * @param y the y position
  * @return draw info to be used on next line
  */
  final synchronized TextDrawInfo drawLine (
    Position p, Position ee, TextDrawInfo drawInfo, int x, int y
  ) {
    if (imGraphics == null) return drawInfo;
    if (ee != null && ee.isEndOfLine ()) ee = null;


    int masc = editor.info.getMaxAscent ();
    Dimension d = editor.info.getCharDimension ();
    boolean showCursor = false;
    try {
      if (cursor != null && p.sameLine (editor.getCursorPos ())) {
        showCursor = true;
        drawCursor ();
      };

      if (drawInfo.end.equals (p)) {
        drawInfo = getTextDrawInfo (drawInfo);
      };

      Position end = ee == null ? p.endOfLine () : Position.min (ee, p.endOfLine ());

      // draw text on line
      if (p.lessOrEqual (end)) for (;;) {

        Position last = end.sameLine (drawInfo.end) ? Position.min (drawInfo.end, end) : end;

        // prepares image to be drawn (first background)
        useTextDrawInfo (drawInfo, false);
        String str = p.toString (last);
        int len = last.getColumn () - p.getColumn ();
        fillRect (x, y - masc, len * d.width, d.height);
        // foreground
        useTextDrawInfo (drawInfo, true);
        drawString (str, x, y);
        // to next position
        x += d.width * len;

        p = last;
        if (p.greaterOrEqual (end)) {
          break;
        };
        drawInfo = getTextDrawInfo (drawInfo);
      };

      Position stop = p.isEndOfLine () ? p : p.endOfLine ();

      while (drawInfo.end.lessOrEqual (p)) {
        drawInfo = getTextDrawInfo (drawInfo);
      };

      // draw position till the last of line
      while (p.sameLine (drawInfo.end) && p.less (drawInfo.end)) {
        Position next = p.sameLine (drawInfo.end) ? drawInfo.end : p.lastOfLine ();
        Position last = ee == null ? next : Position.min (ee, next);

        if (p.less (last) || last.isEndOfLine ()) {
          int len = last.getColumn () - p.getColumn ();
          // prepares image to be drawn (first background)
          useTextDrawInfo (drawInfo, false);
          fillRect (x, y - masc, len * d.width, d.height);
        };
        // to next position
        x += d.width * (next.getColumn () - p.getColumn ());
        p = next;

        if (p.sameLine (drawInfo.end)) {
          if (stop.less (drawInfo.end)) {
            stop = drawInfo.end;
          };
          drawInfo = getTextDrawInfo (drawInfo);
        };
      };

      x = d.width * (stop.getColumn () - firstColumn);
      useTextDrawInfo (drawInfo, false);
    } catch (InvalidPositionException ex) {
      // thrown when getTextDrawInfo returns less than the position asked
      setColor (editor.getBackground ());
    } finally {
      fillRect (x, y - masc, editor.size ().width, d.height);
      if (showCursor) drawCursor ();
    };
    return drawInfo;
  };

  /** Draws a part of text. On one line.
  * @param p the position to begin display from
  * @return draw info to be used on next line
  */
  final synchronized TextDrawInfo drawLine (Position p) {
    try {
      StartDrawInfo sdi = editor.info.beginDraw (p);
      Point xy = positionToPoint (sdi.first);
      return drawLine (sdi.first, sdi, xy.x, xy.y);
    } catch (InvalidPositionException ex) {
      return null;
    }
  }


  /** Draws columns.
  * @param beg the first line to draw
  * @param count number of lines to draw
  * @param from the first column to draw
  * @param to the last column to draw
  */
  final synchronized void drawColumns (Position beg, int count, int from, int to) {
    beg = beg.beginOfLine ();
    try {
      Point p = positionToPoint (beg.column (from));
      Dimension d = editor.info.getCharDimension ();
      while (count-- > 0) {
        Position f = beg.column (from);
        Position t = beg.column (to);
        StartDrawInfo di = editor.info.beginDraw (f);
        drawLine (di.first, t, di, p.x, p.y);
        beg = beg.down ();
        p.y += d.height;
      };
    } catch (EndOfTextException ex) {
    } catch (InvalidPositionException ex) {
    };
  };

  /** Draws a text from position p from coordinates x, y.
  * @param p the position to begin the draw
  * @param end the last position to draw
  */
  final synchronized void draw (Position p, Position end) {
    try {
      TextDrawInfo di = null;
      Point point = null;
      Dimension d = editor.info.getCharDimension ();
      try {
        if (end != null && p.sameLine (end)) {
          // draw the rest of line
          StartDrawInfo sdi = editor.info.beginDraw (p);
          point = positionToPoint (sdi.first);
          di = drawLine (sdi.first, end, sdi, point.x, point.y);
        } else {
          // draw the rest of line
          StartDrawInfo sdi = editor.info.beginDraw (p);
          point = positionToPoint (sdi.first);
          di = drawLine (sdi.first, sdi, point.x, point.y);

          // draw line inside
          int lines = end == null ? visibleLinesCount () : lineToInt (end) - 1;
          for (int i = lineToInt (p); i < lines; i++) {
            point.y += d.height;
            p = p.beginOfLine ().down ().column (firstColumn);
            di = drawLine (p, di, 0, point.y);
          };
          if (end != null) {
            // draw the rest of line
            point.y += d.height;
            p = p.beginOfLine ().down ().column (firstColumn);
            di = drawLine (p, end, di, 0, point.y);
          };
        };
        useTextDrawInfo (di, false);
      } catch (InvalidPositionException ex) {
        setColor (editor.getBackground ());
      } catch (EndOfTextException ex) {
        setColor (editor.getBackground ());
      };

      if (end == null && point != null) {
        Dimension dim = editor.size ();
        fillRect (0, point.y - editor.info.getMaxAscent (), dim.width, dim.height);
      };
    } catch (NullPointerException ex) {
    };
  };

  /** Draws whole text
  */
  final synchronized void draw () {
    draw (getBegin (), null);
  };

  /** Scrolls up one line. Moves everything down and draws new line at
  * the beginning.
  * @exception EndOfTextException if we are already at the begining of text
  */
  final synchronized void scrollUp () throws EndOfTextException {
    try {
      drawCursor ();

      // can throw exception
      begin = getBegin ().up ();

      int lh = editor.info.getCharDimension ().height;

      Dimension dim = editor.size ();
      copyArea (0, 0, dim.width, dim.height - lh, 0, lh);

      drawLine (begin);
    } finally {
      drawCursor ();
    };
  };

  /** Scrolls down one line. Moves everything up and draws new line at the end
  * @exception EndOfTextException if we are already at the end of text
  */
  final void scrollDown () throws EndOfTextException {
    try {
      // hides cursor
      drawCursor ();

      begin = getBegin ().down ();

      int ln = editor.info.getCharDimension ().height;
      Dimension dim = editor.size ();
      copyArea (
        0, ln, dim.width, dim.height - ln, 0, -ln
      );

      setColor (editor.getBackground ());
      fillRect (0, dim.height - ln, dim.width, ln);

      drawLine (getEnd ().up ().beginOfLine ());
      drawLine (getEnd ().beginOfLine ());
    } finally {
      // displays cursor
      drawCursor ();
    };
  };

  /** Locks redrawing.
  */
  public final synchronized void redrawLock (Document d) {
    if (redrawLock == 0) {
      redrawDelayed = false;
    };
    redrawLock++;
  };
  /** Unlocks redrawing.
  */
  public final synchronized void redrawUnlock (Document d) {
    if (--redrawLock <= 0) {
      redrawLock = 0;
      if (redrawDelayed) {
        // if the cursor is drawn than it is hiden
        if (cursor != null) drawCursor ();
        draw ();
        drawCursor ();
      };
      editor.repaint ();
    };
  };
  
  /** Redraws a block
  * @param b the block to redraw
  */
  public final void redrawBlock (Block b) {
    setDefaultBackground ();
    if (!b.isEmpty () || b.getBegin ().isEndOfLine ()) {
      try {
        // hide cursor
        drawCursor ();
        Block n = b.intersection (this);

        if (!n.isEmpty () || b.getBegin ().isEndOfLine ()) {
          draw (n.getBegin (), n.getEnd ());
        };
      } finally {
        // show cursor
        drawCursor ();
      };
    };
  };

  /** Sets default background.
  */
  private final void setDefaultBackground () {
    Color c = editor.info.getDefaultBackground ();
    if (c != null) {
      editor.setBackground (c);
    };
  };

  /** Is the cursor visible? If it is not then screen is shifted.
  * @return true if repositioned and redrawn
  */
  final boolean testVisible () {
    if (!containsOrBegins (editor.getCursorPos ())) {
      setBegin (editor.getCursorPos ().safeUp (visibleLinesCount () / 2));
      return true;
    } else {
      return false;
    }
  }

  /** Notifies that an insert has been done at position p.
  * @param p the position where the insert occures (same as the position
  *   sent to allowInsert)
  * @param n the position of newly inserted character(s) (this is before p)
  */
  public void notifyInsert (Position n, Position p) {
    if (p.equals (getBegin ())) {
      if (redrawLock != 0) {
        redrawDelayed = true;
        return;
      };

      try {
        scrollUp ();
      } catch (EndOfTextException ex) {
      };
    } else {
      if (containsOrBeginsOrEnds (n)) {
        if (redrawLock != 0) {
          redrawDelayed = true;
          return;
        };

        synch ();
        try {
          StartDrawInfo sdi = editor.info.beginInsert (n, p);
          Point pos = positionToPoint (sdi.first);
          Dimension dim = editor.info.getCharDimension ();
          Dimension dimen = editor.size ();
          int masc = editor.info.getMaxAscent ();
          if (sdi.first.sameLine (sdi.last)) {
            // cursor off
            drawCursor ();

            // number of bytes that were really inserted
            int len = p.getColumn () - n.getColumn ();
            copyArea (pos.x, pos.y - masc, dimen.width, dim.height, len * dim.width, 0);

            drawLine (sdi.first, sdi.last, sdi, pos.x, pos.y);
            // cursor on
            drawCursor ();
          } else {
            if (n.toString (p).equals ("\n")) {
              drawCursor ();
              try {
                Point point = positionToPoint (n);

                copyArea (0, point.y - masc + dim.height, dimen.width, dimen.height, 0, dim.height);
                copyArea (point.x, point.y - masc, dimen.width, dim.height, -point.x, dim.height);

                TextDrawInfo di = drawLine (sdi.first, sdi, pos.x, pos.y);
                point = positionToPoint (p);
                drawLine (p, sdi.last, di, point.x, point.y);
              } finally {
                drawCursor ();
              };
            } else {
              // if cursor is not visible, than whole text is redrawn
              if (testVisible ()) return;

              drawCursor ();
              draw (sdi.first, null);
              drawCursor ();
            };
          };
        } catch (InvalidPositionException ex) {
          testVisible ();
        };
      };
    };
  };

  /** Notifies a delete
  * @param p the position where a delete occures
  */
  public void notifyDelete (Position p, String s) {
    if (containsOrBegins (p)) {
      if (redrawLock != 0) {
        redrawDelayed = true;
        return;
      };

      synch ();
      try {
        drawCursor ();

        StartDrawInfo sdi = editor.info.beginDelete (p, s);
        if (s.length () != 1) {
          draw (sdi.first, null);
        } else {
          if (s.equals ("\n")) {
            if (p.sameLine (getBegin ())) {
              draw ();
            } else {
              Dimension dim = editor.info.getCharDimension ();
              Position beg = p.beginOfLine ();
              Point q = positionToPoint (beg);
              Point point = positionToPoint (sdi.first);
              int masc = editor.info.getMaxAscent ();

              int h = dim.height * 2;
              Dimension dimen = editor.size ();
              copyArea (0, q.y - masc + h,
                dimen.width, dimen.height, 0, -dim.height
              );

              drawLine (sdi.first, null, sdi, point.x, point.y);

              // draw the last line
              try {
                p = getBegin ().down (visibleLinesCount () - 1);
                draw (p, null);
              } catch (EndOfTextException ex) {
              };
            };
          } else {
            // delete of normal character
            Dimension dim = editor.info.getCharDimension ();
            Point q = positionToPoint (p);
            Point point = positionToPoint (sdi.first);
            int masc = editor.info.getMaxAscent ();

            Dimension dimen = editor.size ();
            copyArea (q.x + dim.width, q.y - masc, dimen.width, dim.height, -dim.width, 0);

            drawLine (sdi.first, sdi.last, sdi, point.x, point.y);

            // redraw the last part of line (that has not been visible)
            try {
              Position from = p.column ((dimen.width - 1)/ dim.width).previous ();
              if (!from.isEndOfLine ()) {
                drawLine (from);
              };
            } catch (EndOfTextException ex) {
            };
          };
        };
      } catch (InvalidPositionException ex) {
      } finally {
        drawCursor ();
      };
    };
  };
};








