/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.awt.Event;
import java.util.Enumeration;
/**
* This class is used as a handle to opened document. When this handle is
* lost the document is assumed to be closed.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class Source extends Object {
  LoadedDocument doc;



  /** Creates new document.
  * @param d the document
  */
  Source (LoadedDocument d) {
    (doc = d).lockSource ();
  };

  /** Creates new source.
  * @param xf the xelfi file of the class.
  * @param create create new class if it does not exist?
  * @exception IOException if we should not create and the file cannot be loaded
  */
  public Source (XelfiFile xf, boolean create) throws IOException {
    Editor.doStaticOpen (xf, this, create);
    doc.lockSource ();
    doc.setModified (false);
  };

  /** Creates new source.
  * @param xf the xelfi file of the class.
  * @exception IOException if an error occured during load
  */
  public Source (XelfiFile xf) throws IOException {
    this (xf, true);
  };

  /** Creates new source for class n.
  * @param n the name of the class
  * @param create create new class if it does not exist?
  * @exception IOException if the class is already compiled but source code
  *   missing (create) or if the source file is missing (do not create).
  */
  public Source (String n, boolean create) throws IOException {
    Editor.doStaticOpen (XelfiFile.find (n, create), this, create);
    doc.lockSource ();
    doc.setModified (false);
  };

  /** Creates new source for class n.
  * @param n the name of the class
  * @exception IOException if the class is already compiled but source code
  *   missing
  */
  public Source (String n) throws IOException {
    this (n, true);
  };

  /** Adds new listener.
  * @param l the listener
  */
  public void addSourceListener (SourceListener l) {
    doc.list.addElement (l);
  };

  /** Removes listener.
  * @param l the listener
  * @return true if l was in the list
  */
  public boolean removeSourceListener (SourceListener l) {
    return doc.list.removeElement (l);
  };

  /** Returns the document for this source.
  * @return the document
  */
  public Document getDocument () {
    return doc;
  };
  
  /** Returns name of the class this documents contains source text for.
  * @return the class name
  */
  public String getName () {
    return doc.className;
  };

  /** Returns source file.
  * @return the source file
  */
  public File getSource () {
    return doc.fileName;
  };

  /** Returns the xelfi file of this class.
  * @return the file
  */
  public XelfiFile getFile () {
    return doc.xelfiFile;
  };

  /** Saves this source to disk.
  * @exception IOException if an error occures
  */
  public void save () throws IOException {
    if (isModified ()) {
      doc.handler.saveSource (this, Editor.backupFiles);
      doc.setModified (false);
      Enumeration en = doc.list.elements ();
      while (en.hasMoreElements ()) {
        ((SourceListener)en.nextElement ()).sourceSaved (this);
      };
    };
  };

  /** Test if this document has been modified.
  * @return true if it has been
  */
  public boolean isModified () {
    return doc.isModified ();
  };

  /** Marks the source to be modified.
  */
  public void markModified () {
    doc.setModified (true);
  };

  /** Clears the source's modified flag.
  */
  public void clearModified () {
    doc.setModified (false);
  };

  /** Reloads class. Deletes contents of class and loads new one from specified filename.
  * A null info will be sent to allowInsert and allowDelete.
  * @param name name of file to load from
  * @exception IOException if it is not possible to load file
  * @exception NotAllowedException if action is not allowed
  */
  public void reload (File name) throws IOException, NotAllowedException {
    reload (name, null);
  };

  /** Reloads class. Deletes contents of class and loads new one from specified filename.
  * @param name name of file to load from
  * @param info the info to be sent to allowInsert and allowDelete
  * @exception IOException if it is not possible to load file
  * @exception NotAllowedException if action is not allowed
  */
  public void reload (File name, Object info) throws IOException, NotAllowedException {
    Position p = doc.getBegin ();
    p.insert (new DataInputStream (new FileInputStream (name)), info);
    p.deleteTill (doc.getEnd (), info);
  };

  
  /** Removes one in counter of references to this document */
  public void finalize () {
    if (doc != null) {
      doc.unlockSource (this);
      doc = null;
    };
  };

  /** Two sources are equal if they access the same document.
  * @param s the second source
  * @return true if sources are equal
  */
  public boolean equals (Object s) {
    return (s instanceof Source) && (doc == ((Source)s).doc);
  };

  /** Hashes the source.
  * @return the hash code
  */
  public int hashCode () {
    return doc.hashCode ();
  };

  /** Sets new document handler to the document indicated by
  * this source.
  * @param dh the document handler
  */
  public void setHandler (DocumentHandler dh) {
    doc.handler = dh;
  };

  /** Gets the handler for associated document
  * @return the handler
  */
  public DocumentHandler getHandler () {
    return doc.handler;
  };
  
  /** Shows all views. Asks document hander to show all views.
  */
  public void showAll () {
    doc.handler.showAll (this);
  };
};



