/** TopWindowBase - common base functionality support for TopFrame and TopDialog classes.
  @author Filip Dvorak
  @version 1.0
*/

package xelfi.top;

import java.awt.*;
import java.io.File;
import java.util.*;

import xelfi.design.forms.XelfiDesignForm;
import xelfi.xedit.JavaEditorFrame;
import xelfi.editor.*;
import xelfi.top.visual.EditorMenuItem;
import xelfi.util.MessageBox;

import xelfi.compiler.*;  // shortcuts to next and prev. errors

class TopWindowBase
{
    /* id and focusSwitching is set up by TopLevel
     */
  int id;
  boolean focusSwitching;

  private boolean registered; // to be sure that we don't unregister twice

    /** If this variable is true this frame was not shown yet. */
  boolean firstShow;  // to call TopOptions.setupWindow()

    /* Folowing variables are valid only in JavaEditorFrame, we know that it inherits from TopFrame and not from TopDialog  */
  private EditorMenuItem editorMenuItem;
  private static  boolean createSeparator = true;  // separator in the Window menu

  private Vector focusedItems;

  private Window window;

  TopWindowBase(TopFrame w)
  {
    window = (Window)w;
    commonInit();
  }

  TopWindowBase(TopDialog w)
  {
    window = (Window)w;
    commonInit();
  }

  private void commonInit()
  {
      // registarce
    TopLevel.register(this);
    registered = true;

    initFocusedItems();
    firstShow = true;

      // specials
    if(id == TopLevel.W_EDITOR) window.setBackground(Color.lightGray);
  }

  void initFocusedItems()
  {
    focusedItems = new Vector(6);
    addFocusedItem(MainMenu.miWindowClose);
    if(id == TopLevel.W_EDITOR)
    {
      addFocusedItem(MainMenu.miDebugGoCursor);
      addFocusedItem(MainMenu.miDebugToggleBreak);
      addFocusedItem(MainMenu.miSystemSave);
      addFocusedItem(MainMenu.miSystemSaveTemplate);
      addFocusedItem(MainMenu.miEditUndo);
      addFocusedItem(MainMenu.miEditCut);
      addFocusedItem(MainMenu.miEditCopy);
      addFocusedItem(MainMenu.miEditPaste);
      addFocusedItem(MainMenu.miEditGoTo);
      addFocusedItem(MainMenu.miEditFind);
      addFocusedItem(MainMenu.miRunSetMain);
    }
    else if(id == TopLevel.W_FORM_FRAME || id == TopLevel.W_FORM_DIALOG ||
      id == TopLevel.W_FORM_APPLET)
    {
      addFocusedItem(MainMenu.miSystemSave);
      addFocusedItem(MainMenu.miSystemSaveTemplate);
      addFocusedItem(MainMenu.miRunSetMain);
    }
  }

  /** Handles focus switching, and common shortcut keys */
  public boolean handleEvent(Event ev)
  {
    // handle some special situations
    if(ev.target == window && ev.id == Event.WINDOW_DESTROY)
      switch(id)
      {
        case TopLevel.W_EDITOR:
          window.hide();
          window.dispose();
          return true;
        default:
          window.hide();
          return true;
      }
    return false;
  }

  void beforeShow()
  {
      // setup window shape
    if(firstShow)
    {
      firstShow = false;
      Desktop.setupWindow(this);
    }

      // check Window minu item
    int toCheck = -1;
    switch(id)
    {
      case TopLevel.W_WATCHES: toCheck = MainMenu.miWindowWatches; break;
      case TopLevel.W_LOCALS: toCheck = MainMenu.miWindowLocals; break;
      case TopLevel.W_CALLSTACK: toCheck = MainMenu.miWindowCallStack; break;
      case TopLevel.W_THREADS: toCheck = MainMenu.miWindowThreads; break;
      case TopLevel.W_BREAKPOINTS: toCheck = MainMenu.miWindowBreakpoints; break;
      case TopLevel.W_OUTPUT: toCheck = MainMenu.miWindowOutput; break;
      case TopLevel.W_INSPECTOR: toCheck = MainMenu.miWindowInspector; break;
    }
    if(toCheck != -1) ((CheckboxMenuItem)TopLevel.getMenu().getItem(toCheck)).setState(true);

      // create menu item for Editor
    if(id == TopLevel.W_EDITOR && editorMenuItem == null)
    {
      JavaEditorFrame me = (JavaEditorFrame)window;
      editorMenuItem = new EditorMenuItem(me);
      editorMenuItem.setState(true);
    }
  }

  void afterHide()
  {
      // Uncheck Window menu item
    int toUncheck = -1;
    switch(id)
    {
      case TopLevel.W_WATCHES: toUncheck = MainMenu.miWindowWatches; break;
      case TopLevel.W_LOCALS: toUncheck = MainMenu.miWindowLocals; break;
      case TopLevel.W_CALLSTACK: toUncheck = MainMenu.miWindowCallStack; break;
      case TopLevel.W_THREADS: toUncheck = MainMenu.miWindowThreads; break;
      case TopLevel.W_BREAKPOINTS: toUncheck = MainMenu.miWindowBreakpoints; break;
      case TopLevel.W_OUTPUT: toUncheck = MainMenu.miWindowOutput; break;
      case TopLevel.W_INSPECTOR: toUncheck = MainMenu.miWindowInspector; break;
    }
    if(toUncheck != -1) ((CheckboxMenuItem)TopLevel.getMenu().getItem(toUncheck)).setState(false);

      // delete menu item for Editor
    if(id == TopLevel.W_EDITOR && editorMenuItem != null)
    {
       editorMenuItem.removeSelf();
    }
  }

  /** Handles focus switchinhg. Strategy is: focused is currently focused window. LastFocused is
    window that has last released the focus. Menu items are disabled when new window gets the
    focus over lastFocused window. */
  public boolean gotFocus(Event ev, Object what)
  {
//    System.out.println("GotFocus, id = ");
//    System.out.print(id);
//    System.out.println(", focus switching = " + focusSwitching );

      // check EditorMenuItem
    if(id == TopLevel.W_EDITOR && editorMenuItem != null) editorMenuItem.setState(true);

    if(!focusSwitching) return false;

      // switch off menu items of previous window
    if(TopLevel.focused == null)
    {
      if(TopLevel.lastFocused != null)
        switchMenuItems(TopLevel.lastFocused, false);
    }
    else
    {
      switchMenuItems(TopLevel.focused, false);
      TopLevel.lastFocused = TopLevel.focused;
    }

    TopLevel.focused = this;
    switchMenuItems(this, true);

    return true;
  }

  /** Handles focus switchinhg. */
  public boolean lostFocus(Event ev, Object what)
  {

//    System.out.print("LostFocus, id = ");
//    System.out.print(id);
//    System.out.println(", focus switching = " + focusSwitching );

    if(!focusSwitching) return false;

    if(TopLevel.focused == this)
    {
      TopLevel.focused = null;
      TopLevel.lastFocused = this;
    }
    return true;
  }


    /** Key shortcuts */
  public boolean keyDown(Event  e, int  key)
  {
//    System.out.println("toplevel keydown handling");
    boolean ctrl = (e.modifiers & Event.CTRL_MASK) != 0;
    boolean shift = (e.modifiers & Event.SHIFT_MASK) != 0;
    MainMenu menu = TopLevel.getMenu();

    switch(e.key)
    {
      case Event.F3:
        if(id == TopLevel.W_EDITOR) menu.itemAction(MainMenu.miEditFind);
        else return false;
        break;
      case Event.F4:
        menu.itemAction(MainMenu.miDebugGoCursor);
        break;
      case Event.F5:
        if (!shift & !ctrl)
          menu.itemAction(MainMenu.miDebugGo);
        else if (ctrl)
          menu.itemAction(MainMenu.miDebugFinish);
        else if (shift)
          menu.itemAction(MainMenu.miDebugRestart);
        break;
      case Event.F7:
        menu.itemAction(MainMenu.miDebugTraceInto);
        break;
      case Event.F8:
        menu.itemAction(MainMenu.miDebugTraceOver);
        break;
      case Event.F9:
        if (!ctrl)
          menu.itemAction(MainMenu.miDebugToggleBreak);
        else
          menu.itemAction(MainMenu.miDebugAddWatch);
        break;
      case Event.F11:  // switch to Editor
        if(id == TopLevel.W_FORM_FRAME || id == TopLevel.W_FORM_DIALOG || id == TopLevel.W_FORM_APPLET)
          TopLevel.showEditor(((XelfiDesignForm)window).getSource(), true);
        else return false;
        break;
      case Event.F12:  // switch to form
        if(id == TopLevel.W_EDITOR) TopLevel.showForm(((JavaEditorFrame)window).getSource(), true);
        break;
      case Event.F10:
      case 9: //Ctrl+Shift+I
        if (ctrl && shift) menu.itemAction(MainMenu.miWindowInspector);
        break;
      case 23: //Ctrl+Shift+W
        if (ctrl && shift) menu.itemAction(MainMenu.miWindowWatches);
        break;
      case 12:
        if(ctrl && shift) menu.itemAction(MainMenu.miWindowLocals);  //Ctrl+Shift+L
        else if (ctrl) menu.itemAction(MainMenu.miEditGoTo); // //Ctrl+L
        break;
      case 19: //Ctrl+Shift+S
        if(shift && ctrl) menu.itemAction(MainMenu.miWindowCallStack);
        break;
      case 20: //Ctrl+Shift+T
        if (ctrl && shift) menu.itemAction(MainMenu.miWindowThreads);
        break;
      case 2: //Ctrl+Shift+B
        if (ctrl && shift) menu.itemAction(MainMenu.miWindowBreakpoints);
        break;
      case 15: //Ctrl+Shift+O
        if (ctrl && shift) menu.itemAction(MainMenu.miWindowOutput);
        break;
      case 5:  // Ctrl+E
        if(id == TopLevel.W_EDITOR)
        {
          JavaEditorFrame ed = (JavaEditorFrame)window;
          ErrorList ls = ErrorDatabase.getErrorList(ed.getSource().getName());
          ErrorDescription err = null;
          if(ls != null && (err = ls.getLastHighlightedError()) != null)
          {
            ls.setHighlightedError(err);
            err.scroll(ed, true);
          }
        }
        else if(id == TopLevel.W_OUTPUT) ((Output)window).clear();
        else return false;
        break;
      case 1:  // Ctrl+A
        if(id == TopLevel.W_EDITOR)
        {
          JavaEditorFrame ed = (JavaEditorFrame)window;
          ErrorList er = ErrorDatabase.getErrorList(ed.getSource().getName());
          if(er != null) er.previousHighlightedError().scroll(ed, true);
        }
        break;
      case 26:  // Ctrl+Z
        if(id == TopLevel.W_EDITOR)
        {
          JavaEditorFrame ed = (JavaEditorFrame)window;
          ErrorList er = ErrorDatabase.getErrorList(ed.getSource().getName());
          if(er != null) er.nextHighlightedError().scroll(ed, true);
        }
        break;
      case 24: //Ctrl+X
        menu.itemAction(MainMenu.miSystemExit);
        break;
      default: return false;
    }
    return true;
  }

  /** Should be called before closing the frame. (It unregisters the frame) */
  public void done()
  {
      // If disposing window we must explicitly give up the focus.
    if(focusSwitching)
    {
      if(TopLevel.getFocusedBase() == this) switchMenuItems(this, false);  // switch off menu items
      if(TopLevel.focused == this) TopLevel.focused = null;
      if(TopLevel.lastFocused == this) TopLevel.lastFocused = null;
    }

    if(registered)
    {
      registered = false;
      TopLevel.unregister(this);
    }
  }

  static void switchMenuItems(TopWindowBase b, boolean enable)
  {
    MainMenu menu = TopLevel.getMenu();
    Enumeration items = null;

    items = b.focusedItems.elements();
    while(items.hasMoreElements())
        menu.enableItem(((Integer)items.nextElement()).intValue(), enable);
  }

  protected void addFocusedItem(int itemId)
  {
    focusedItems.addElement(new Integer(itemId));
  }

  protected void removeFocusedItem(int itemId)
  {
    focusedItems.removeElement(new Integer(itemId));
  }

  Window getWindow()
  {
    return window;
  }
}





