/** Message box for Xelfi.
*
* To be informed about the type of clicked button the parent
* frame must redefine its <CODE>action</CODE> method following event
* can be catched there:
* <pre>
*   Event.id = ACTION_EVENT
*   Event.target = the message box
*   Event.key = code of clicked button OK, YES, NO or CANCEL
* </pre>
* The id of clicked button can also be obtained by a call to
* the <CODE>getButton</CODE> method.
*
* @author   Filip Dvorak, Jaroslav Tulach, Ian Formanek
* @version  1.2 Support for MessageBoxListener interface
* @version  1.1 Support for more buttons, OK, Yes, No, Cancel
* @version  1.0 Support for OK button only
*/

package xelfi.util;

import java.awt.*;
import java.util.*;

import xelfi.top.*;
import xelfi.top.visual.*;

public class MessageBox extends TopDialog {
  /** constant for message box's buttons */
  public static final int OK = 0x0001;
  /** constant for message box's buttons */
  public static final int YES = 0x0002;
  /** constant for message box's buttons */
  public static final int NO = 0x0004;
  /** constant for message box's buttons */
  public static final int CANCEL = 0x0008;
  /** displays OK and Cancel buttons */
  public static final int OKCANCEL = OK | CANCEL;
  /** displays Yes, No and Cancel buttons */
  public static final int YESNOCANCEL = YES | NO | CANCEL;
  /** displays Yes and No buttons */
  public static final int YESNO = YES | NO;

  /** the last number in buttons */
  private static final int COUNT = 4;

  /** button names */
  private static Hashtable buttonNames = new Hashtable ();
  /** its initialization */
  static {
    buttonNames.put (new Integer (OK), "OK");
    buttonNames.put (new Integer (CANCEL), "Cancel");
    buttonNames.put (new Integer (YES), "Yes");
    buttonNames.put (new Integer (NO), "No");
  };

  /** the id of clicked button */
  private int button = 0;

  /** table with buttons in this message box */
  private Hashtable buttons = new Hashtable ();

    /** Length of one row in characters */
  public static int MAXROW = 70;

  // MessageBoxListener which will be notified on button press.
  // if null then action event will be sent to parent
  private MessageBoxListener listener = null;

  /** Constructs message box with only the OK button. 
  * MessageBoxListener will be notified on button press.
  * @param caption the title
  * @param message the text to display in the box
  * @param ml      MessageBoxListener which will be notified on button press
  * @see MessageBoxListener
  */
  public MessageBox(String caption, String message, MessageBoxListener ml)
  {
    this(TopLevel.getMenu(), caption, message, OK);
    listener = ml;
  }

  /** Constructs message box with only the OK button.
  * MessageBoxListener will be notified on button press.
  * @param caption the title
  * @param message the text to display in the box
  * @param butt the ORed constants OK, CANCEL, YES, NO
  * @param ml      MessageBoxListener which will be notified on button press
  * @see MessageBoxListener
  */
  public MessageBox(String caption, String message, int butt, MessageBoxListener ml)
  {
    this(TopLevel.getMenu(), caption, message, butt);
    listener = ml;
  }

  /** Constructs message box with only the OK button.
  * @param caption the title
  * @param message the text to display in the box
  */
  public MessageBox(String caption, String message)
  {
    this(TopLevel.getMenu(), caption, message, OK);
  }

  /** Constructs message box with only the OK button.
  * @param caption the title
  * @param message the text to display in the box
  * @param butt the ORed constants OK, CANCEL, YES, NO
  */
  public MessageBox(String caption, String message, int butt)
  {
    this(TopLevel.getMenu(), caption, message, butt);
  }
  
  /** Constructs message box with only the OK button.
  * @param parent the parent frame
  * @param caption the title
  * @param message the text to display in the box
  */
  public MessageBox(Frame parent, String caption, String message) {
    this (parent, caption, message, OK);
  };

  /** Constructs message box.
  * @param parent the parent frame
  * @param caption the title
  * @param message the text to display in the box
  * @param butt the ORed constants OK, CANCEL, YES, NO
  */
  public MessageBox(Frame parent, String caption, String message, int butt)
  {
    super(parent, caption, true);
    
    Vector labels = parseText(message);

    setResizable(true);
    RowLayout rl = new RowLayout();
    setLayout(rl);
    RowLayoutProperties pr = new RowLayoutProperties();

    int last = labels.size() - 1;
    for(int i = 0; i <= last; i++)
    {
      Label l = (Label)labels.elementAt(i);
      l.setAlignment(Label.CENTER);
      if(i == 0)
      {
        pr.topSpaceGrowth = 0.3f;
        if(last == 0) pr.compInsets = new Insets(2, 2, 2, 6);
        else pr.compInsets = new Insets(2, 2, 0, 6);
      }
      else
      {
        pr.topSpaceGrowth = 0.0f;
        pr.compInsets = new Insets(0, 2, 0, 6);
      }
      if(i == last) pr.bottomSpaceGrowth = 0.52f;
      else pr.bottomSpaceGrowth = 0.0f;
      add(l);
      rl.add(l, pr);
    }

    Panel p = new Panel ();
    p.setLayout (new FlowLayout ());
    pr.topSpaceGrowth = 0.0f;
    pr.bottomSpaceGrowth = 0.15f;
    pr.compGrowth = 0.03f;
    pr.leftSpaceGrowth = 0.46f;
    pr.rightSpaceGrowth = 0.46f;
    pr.compInsets = new Insets(14, 2, 2, 2);

    for (int i = 0, c = 0x0001; i < COUNT; i++, c *= 2) {
      if ((c & butt) != 0) {
        Integer ci = new Integer (c);
        buttons.put (p.add (new Button ((String)buttonNames.get (ci))), ci);
      };
    };
    
    add (p);
    rl.add(p, pr);

    pack();
    pack();
  }

  private Vector parseText(String txt)
  {
    Vector labels = new Vector(8);
    StringTokenizer tok = new StringTokenizer(txt, "\n\t ", true);

    int count = 0;
    boolean flag = true;
    String s = null;
    while(tok.hasMoreTokens())
    {
      StringBuffer buf = new StringBuffer((int)(MAXROW * 1.2));
      if(!flag)
      {
        buf.append(s);
        flag = true;
        count = 1;
      }
      else count = 0;

      while(tok.hasMoreTokens())
      {
        s = tok.nextToken();
        flag = false;
        if(!s.equals("\n"))
        {
          if(count == 0 || buf.length() + s.length() < MAXROW)
          {
            buf.append(s);
            flag = true;
          }
          if(buf.length() + s.length() > MAXROW) break;
        }
        else  // s == \n
        {
          flag = true;
          break;
        }
      }
//      System.out.println("adding " +  buf.toString());
      labels.addElement(new Label(buf.toString()));
    }
    return labels;
  }

  public void show()
  {
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();

    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
  }

  public boolean action(Event e, Object arg)
  {
    Integer click = (Integer)buttons.get (e.target);
    if (click != null) {
      hide();

      button = click.intValue ();
      if (listener != null)
         listener.messageNotify(button);
      else {
        //return true;
        // sets the constant for this button to key field of event and
        // continues with propagation of the event
        // target is also set to this field, so the parent can test for it
        Event ev = new Event (this, Event.ACTION_EVENT, null);
        ev.key = button;
        getParent ().deliverEvent (ev);
      }
      return true;
    }
    return super.action(e, arg);
  }

  public boolean handleEvent(Event event)
  {
    if (event.id == Event.WINDOW_DESTROY)
    {
      hide();
      dispose();
     return true;
    }
    return super.handleEvent(event);
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }

  /** The clicked button.
  * @return the code of clicked button (OK, ...) or 0 if nothing has been clicked
  */
  public int getButton () {
    return button;
  };
}
















