/**
* Xelfi programming environment
*/

package xelfi.xedit;
import xelfi.editor.Acceptor;
import xelfi.editor.Position;
import xelfi.editor.Block;
import xelfi.editor.EndOfTextException;
import xelfi.editor.InvalidPositionException;

/**
* Looks for chars / and * or * and / and "
*
* It scans only (modified) block passed in constructor and
* character before and after.
* findNext should stop only at new occurences of special characters.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class CommentsAcceptor extends Object implements Acceptor {
  /** type of lastly founded character */
  byte type;

  /** current position for scanning */
  private Position current;

  /** end position */
  private Position end;

  /** last character */
  private char last = 0;

  /** Acceptor searches for ", /* and * / and //
  * @param b the block to search in, this block has been changed => we
  *   must scan also one character before and after
  */
  public CommentsAcceptor (Block b) {
    current = b.getBegin ();
    end = b.getEnd ();
    try {
      Position p;
      // store the previous symbol
      last = (p = current.previous ()).getChar ();

      char ch = p.previous ().getChar ();
      if (last == '/' && (ch == '*' || ch == '/')) {
        // the / is end of previous symbol, forget it
        last = 0;
      }
      if (last == '*' && ch == '/') {
        // the * is end of previous comment, forget it
        last = 0;
      }
    } catch (EndOfTextException ex) {
    }
  }

  /** Create mark.
  * @return the mark
  */
  private CommentMark returnMark () {
    switch (type) {
    case CommentMark.BEGIN:
    case CommentMark.LINE:
      // the previous () call in next line should never fall
      try {
        return new CommentMark (current.previous (), type);
      } catch (EndOfTextException ex) {
        return null;
      }
    case CommentMark.STRING:
      // skip the " and go to defaults
      Position c = current;
      current = current.next ();
      return new CommentMark (c, type);
    case CommentMark.END:
    default:
      return new CommentMark (current, type);
    }
  }

  /** Look for next important symbol. Searches the block passed in the constructor.
  * @return the info about it or null if nothing has been found
  */
  public CommentMark findNext () {
    // if end is null then end of scanned text has been found => return null
    if (end == null) return null;
    CommentMark toReturn = null;
    try {
      current = new Block (current, end).find (this);
      toReturn = returnMark ();
    } catch (EndOfTextException ex) {
      // nothing has been found, test the next character
      char ch = end.getChar ();
      // to signal end of searching
      try {
        if (ch != '\"' && accept (ch)) {
          // this last char is important
          current = end;
          toReturn = returnMark ();
        }
      } finally {
        end = null;
      }
    } catch (InvalidPositionException ex) {
      // current is not before
    } finally {
      // at every case clear the last variable to solve cases like:
      // "/*/" be accepted twice once like "/*" and second time as "*/"
      last = 0;
    }
    // nothing found
    return toReturn;
  }

  /**
  * should we continue or stop the action?
  * @param ch the character at specified position
  * @return true/false
  */
  public boolean accept (char ch) {
    try {
      switch (ch) {
      case '\"':
        // string, test if the previous char has not been backslash
        if (last != '\\') {
          type = CommentMark.STRING;
          return true;
        }
        break;
      case '*':
        if (last == '/') {
          type = CommentMark.BEGIN;
          return true;
        }
        break;
      case '/':
        if (last == '*') {
          type = CommentMark.END;
          return true;
        }
        if (last == '/') {
          type = CommentMark.LINE;
          return true;
        }
        break;
      }
      return false;
    } finally {
      last = ch;
    }
  }
}












