/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.awt.*;
import java.io.*;
import java.net.*;
import xelfi.editor.*;

/**
* This is class for simple text editor.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class EditorFrame extends Frame {
  /** the editor */
  TextEditor editor;

  /** the goto line dialog */
  private DialogGoToLine goTo;

  /** find dialog */
  private DialogFind find;

  /** the filename */
  private String file;

  /** menu items */
  private MenuItem mi_new;
  private MenuItem mi_load;
  private MenuItem mi_save;
  private MenuItem mi_saveAs;
  private MenuItem mi_fork;
  private MenuItem mi_quit;
  private MenuItem mi_close;
  private MenuItem mi_copy;
  private MenuItem mi_cut;
  private MenuItem mi_clear;
  private MenuItem mi_paste;
  private MenuItem mi_find;
  private MenuItem mi_goto;

  /** number of opened editors */
  private static int windows = 0;

  /** Opens new empty editor.
  */
  public EditorFrame () {
    this (null);
  };

  /** Creates new editor window for specified file.
  * @param file the filename
  */
  public EditorFrame (String file) {
    this (file, null);
  };

  /** Creates new editor window for specified file.
  * @param file the filename
  * @param doc the document if it is null a new one is loaded
  */
  public EditorFrame (String file, Document doc) {
    super (file == null ? "Java Text Editor" : file);
    doc = doc == null ? openDocument (file) : doc;

    add ("Center", editor = new TextEditor (doc, true));

    goTo = new DialogGoToLine (this, editor);
    find = new DialogFind (this, editor);
    this.file = file;

    createMenu ();

    // Adds syntax callback
//    ((StandardEditorCallback)editor.getEditorCallback ())
//    .changeSyntaxCallback (new SyntaxComments ());


    pack ();
    windows++;
  };

  public static Document openDocument (String file) {
    Document d;
    try {
      InputStream s = openFile (file);
      d = new Document (new DataInputStream (s));
    } catch (Exception ex) {
      d = new Document ();
    };
    return d;
  };

  /** Creates menu. */
  private void createMenu () {
    MenuBar mb = new MenuBar ();
    Menu m;

    m = new Menu ("File");
    m.add (mi_new = new MenuItem ("New"));
    m.add (mi_load = new MenuItem ("Load ..."));
    m.add (mi_save = new MenuItem ("Save"));
    m.add (mi_saveAs = new MenuItem ("Save as ..."));
    m.add (mi_fork = new MenuItem ("Fork"));
    m.addSeparator ();
    m.add (mi_close = new MenuItem ("Close"));
    m.addSeparator ();
    m.add (mi_quit = new MenuItem ("Quit"));
    mb.add (m);

    m = new Menu ("Block");
    m.add (mi_copy = new MenuItem ("Copy"));
    m.add (mi_cut = new MenuItem ("Cut"));
    m.add (mi_clear = new MenuItem ("Clear"));
    m.add (mi_paste = new MenuItem ("Paste"));
    mb.add (m);

    m = new Menu ("Search");
    m.add (mi_find = new MenuItem ("Find ..."));
    m.add (mi_goto = new MenuItem ("Go to line ..."));
    mb.add (m);


    setMenuBar (mb);
  };

  protected static InputStream openFile (String s) throws Exception {
    return new FileInputStream (s);
  };

  /** Handle event.
  * @param ev the event
  */
  public boolean handleEvent (Event ev) {
    try {
      if (!super.handleEvent (ev)) {
        if (ev.id == Event.WINDOW_DESTROY) {
          close ();
          return true;
        };
        return false;
      };
    } catch (NullPointerException ex) {
    };
    return true;
  };
  
  /** redefine key handling method
  * @param e the event that happened
  * @param key the key pressed
  */
  public boolean keyDown (Event e, int key) {
    boolean ctrl = (e.modifiers & Event.CTRL_MASK) != 0;
    
    switch (key) {
    case 6: //Ctrl + F
      showFind ();
      return true;
    };
    return false;
  };


  /** Handle event method.
  */
  public boolean action (Event ev, Object w) {
    Object t = ev.target;
    if (t == mi_quit) {
      System.exit (0);
      return true;
    };
    if (t == mi_close) {
      close ();
      return true;
    };
    if (t == mi_fork) {
      EditorFrame e = new EditorFrame (file, editor.getDocument ());
      e.show ();
      e.editor.setCursorPos (editor.getCursorPos ());
      return true;
    };
    if (t == mi_new) {
      new EditorFrame ().show ();
      return true;
    };
    if (t == mi_load) {
      String s;
      FileDialog fd = new FileDialog (this, "Load");
      fd.show ();
      s = fd.getDirectory () + fd.getFile ();
      if (s != null) {
        new EditorFrame (s).show ();
      };
      return true;
    };
    if (t == mi_saveAs || (t == mi_save && file == null)) {
      FileDialog fd = new FileDialog (this, "Save", FileDialog.SAVE);
      fd.show ();
      file = fd.getDirectory () + fd.getFile ();
      if (file != null) {
        setTitle (file);
      };
      t = mi_save;
    };
    try {
      if (t == mi_save && file != null) {
        save ();
        return true;
      };
  
      if (t == mi_copy) {
        TextEditorOperation.copy (editor);
        return true;
      };
  
      if (t == mi_clear) {
        TextEditorOperation.clear (editor);
        return true;
      };
  
      if (t == mi_cut) {
        TextEditorOperation.cut (editor);
        return true;
      };
  
      if (t == mi_paste) {
        TextEditorOperation.paste (editor);
        return true;
      };
  
      if (t == mi_goto) {
        showGoTo ();
        return true;
      };
  
      if (t == mi_find) {
        showFind ();
        return true;
      };
    } catch (NotAllowedException ex) {
    };

    return false;
  };

  /** Saves the file to disk.
  */
  void save () {
    try {
      OutputStream s = new FileOutputStream (file);

      new DataOutputStream (s).writeBytes (editor.getDocument ().toString ());
    } catch (Exception ex) {
    };
  };

  /** Closes the window */
  private void close () {
    dispose ();
    if (--windows == 0) {
      System.exit (0);
    };
  };

  /** Returns the text editor in this frame.
  * @return the text editor
  */
  public TextEditor getTextEditor () {
    return editor;
  };

  /** Shows dialog for find.
  */
  public void showFind () {
    find.pack ();
    find.show ();
  };

  /** Shows go to line dialog.
  */
  public void showGoTo () {
    goTo.pack ();
    goTo.show ();
  };

};









